use serde::Serialize;

use crate::services::archive::extensions::ArchiveUploadExt;
#[mockall_double::double]
use crate::services::file_store::FileStore;

#[mockall_double::double]
use super::archive::Archive;
#[mockall_double::double]
use super::domain_events::DomainEvents;
#[mockall_double::double]
use super::file_reader::FileReader;

use self::models::{ArtifactID, CommitArtifact};

pub struct ReleaseManager {
    archive: Archive,
    file_reader: FileReader,
    file_store: FileStore,
    domain_events: DomainEvents,
}

impl ReleaseManager {
    pub fn new(
        file_reader: FileReader,
        file_store: FileStore,
        archive: Archive,
        domain_events: DomainEvents,
    ) -> Self {
        Self {
            archive,
            file_reader,
            file_store,
            domain_events,
        }
    }

    pub async fn commit_artifact(&self, request: CommitArtifact) -> anyhow::Result<ArtifactID> {
        tracing::debug!("committing artifact: {:?}", request);
        let artifact_id = ArtifactID::new();

        let files = self.file_reader.read_files(request.folder).await?;

        let archive = self.archive.create_archive(files).await?;

        self.file_store
            .upload_archive(artifact_id.clone(), archive)
            .await?;

        self.domain_events
            .publish_event(&serde_json::to_string(&CommittedArtifactEvent {
                artifact_id: artifact_id.to_string(),
            })?)
            .await?;

        Ok(artifact_id)
    }
}

#[derive(Serialize)]
pub struct CommittedArtifactEvent {
    artifact_id: String,
}

pub mod extensions;
pub mod models;

#[cfg(test)]
mod test {
    use crate::services::archive::{ArchiveFile, MockArchive};
    use crate::services::domain_events::MockDomainEvents;
    use crate::services::file_reader::{Files, MockFileReader};
    use crate::services::file_store::MockFileStore;

    use super::*;

    #[tokio::test]
    async fn generated_artifact_id() -> anyhow::Result<()> {
        let mut file_store = MockFileStore::default();
        file_store
            .expect_upload_file()
            .times(1)
            .returning(|_, _| Ok(()));

        let mut domain_events = MockDomainEvents::default();
        domain_events
            .expect_publish_event()
            .times(1)
            .returning(|_| Ok(()));

        let mut file_reader = MockFileReader::default();
        file_reader
            .expect_read_files()
            .times(1)
            .returning(|_| Ok(Files::default()));

        let mut archive = MockArchive::default();
        archive.expect_create_archive().times(1).returning(|_| {
            Ok(ArchiveFile {
                content: Vec::new(),
            })
        });

        let releaser_manager = ReleaseManager::new(file_reader, file_store, archive, domain_events);

        releaser_manager
            .commit_artifact(CommitArtifact {
                app: "app".into(),
                branch: "branch".into(),
                folder: "someFolder".into(),
            })
            .await?;

        Ok(())
    }
}
